---
tags: [gradio-custom-component, CheckboxGroup, gradio-custom-component, gradio-checkbox-group-markdown]
title: gradio_checkboxgroupmarkdown
short_description: Gradio component for CheckboxGroup with Markdown
colorFrom: blue
colorTo: yellow
sdk: gradio
pinned: false
app_file: space.py
---

# `gradio_checkboxgroupmarkdown`
<a href="https://pypi.org/project/gradio_checkboxgroupmarkdown/" target="_blank"><img alt="PyPI - Version" src="https://img.shields.io/pypi/v/gradio_checkboxgroupmarkdown"></a>  

Gradio component for CheckboxGroup with Markdown

## Installation

```bash
pip install gradio_checkboxgroupmarkdown
```

## Usage

```python
import gradio as gr
import random
from typing import List, Dict
from gradio_checkboxgroupmarkdown import CheckboxGroupMarkdown

def generate_test_articles():
    topics = ["Machine Learning", "Deep Learning", "Neural Networks", "Computer Vision", 
              "Natural Language Processing"]
    
    subtopics = ["Introduction", "Tutorial", "Case Study"]
    
    articles = []
    for i in range(10):
        topic = random.choice(topics)
        subtopic = random.choice(subtopics)
        article_id = f"art_{i+1:02d}"
        
        title = f"{topic}: {subtopic}"
        content = f"""# {title}
This article covers {topic.lower()} {subtopic.lower()}.
Key points:
- Basic concepts
- Implementation tips
- Practical examples"""

        articles.append({
            "id": article_id,
            "title": title,
            "content": content,
            "selected": False
        })
    
    return articles

def search_articles(search_text: str, master_articles: List[Dict]) -> List[Dict]:
    print("search_articles")
    
    """Search articles based on input text"""
    if not search_text.strip():
        return master_articles
    
    search_terms = search_text.lower().split()
    filtered_articles = []
    
    for article in master_articles:
        text_to_search = (article["title"] + " " + article["content"]).lower()
        if all(term in text_to_search for term in search_terms):
            filtered_articles.append(article)
    
    return filtered_articles

def update_filtered_articles(search_text: str, master_articles: List[Dict]):
    print("update_filtered_articles")
    
    """Update the first CheckboxGroupMarkdown with filtered articles"""
    filtered = search_articles(search_text, master_articles)
    
    return {
        filtered_checkbox: gr.update(
            choices=filtered,
            value=[art["id"] for art in filtered if art["selected"]]
        ),
        filtered_checkbox_state: filtered
    }
    
def update_selected_checkbox_articles(selected_choices, filtered_checkbox, master_articles: List[Dict]):
    print("handle_deselect_articles")
    
    """Update master articles by removing unselected ones"""
    # Get IDs of articles that remain selected
    selected_ids = {choice["id"] for choice in selected_choices}
    
    # Update selection status in master_articles
    for article in master_articles:
        article["selected"] = article["id"] in selected_ids
            
    # Update selection status in filtered_checkbox
    for article in filtered_checkbox:
        article["selected"] = article["id"] in selected_ids
        
    # Get selected articles for second tab
    selected_articles = [
        {
            "id": art["id"],
            "title": art["title"],
            "content": art["content"],
            "selected": True
        }
        for art in master_articles 
        if art["selected"]
    ]
    
    return [
        gr.update(
            choices=selected_articles,
            value=[art["id"] for art in selected_articles]
        ),
        gr.update(
            value=[art["id"] for art in filtered_checkbox if art["selected"]]
        ),
        master_articles,
        filtered_checkbox
    ]

def update_filtered_checkbox_articles(selected_choices, filtered_checkbox, master_articles: List[Dict]):
    print("update_selected_articles")
    
    """Update the second CheckboxGroupMarkdown when selections change in the first one"""
    # Get IDs of newly selected articles
    selected_ids = {choice["id"] for choice in selected_choices}
    
    # Update selection status in filtered_checkbox_state
    for article in filtered_checkbox:
        if article["id"] in selected_ids:
            article["selected"] = True
        else:
            article["selected"] = False
            
    # Update selection status in master_articles based on filtered_checkbox
    filtered_articles_dict = {art["id"]: art["selected"] for art in filtered_checkbox}
    for article in master_articles:
        if article["id"] in filtered_articles_dict:
            article["selected"] = filtered_articles_dict[article["id"]]
    
    # Get all selected articles for the second component
    selected_articles = [
        {
            "id": art["id"],
            "title": art["title"],
            "content": art["content"],
            "selected": True
        }
        for art in master_articles 
        if art["selected"]
    ]
    
    return {
        selected_checkbox: gr.update(
            choices=selected_articles,
            value=[art["id"] for art in selected_articles]
        ),
        filtered_checkbox_state: filtered_checkbox,
        master_articles_state: master_articles
    }

# Create the Gradio interface
with gr.Blocks() as demo:
    gr.Markdown("## Article Search and Selection Demo")
    
    # Create state to hold master articles list
    master_articles_state = gr.State(generate_test_articles())
    filtered_checkbox_state = gr.State(master_articles_state.value)
    print("generate articles")
    
    # Search bar
    with gr.Row():
        search_input = gr.Textbox(
            label="Search Articles",
            placeholder="Enter search terms...",
            show_label=True
        )
        search_button = gr.Button("Search")
    
    # Tabs for the two CheckboxGroupMarkdown components
    with gr.Tabs() as tabs:
        with gr.Tab("Search Results"):
            filtered_checkbox = CheckboxGroupMarkdown(
                choices=master_articles_state.value,
                label="Available Articles",
                info="Select articles to add to your collection",
                type="all",
                value=[art["id"] for art in master_articles_state.value if art["selected"]],
                buttons=["select_all", "deselect_all"]
            )
        print("filtered_checkbox")
        
        with gr.Tab("Selected Collection"):
            selected_checkbox = CheckboxGroupMarkdown(
                choices=[art for art in master_articles_state.value if art["selected"]],
                label="Your Selected Articles",
                info="Your curated collection of articles",
                type="all",
                value=[art["id"] for art in master_articles_state.value if art["selected"]],
                buttons=["select_all", "deselect_all"]
            )
        print("selected_checkbox")
    
    # Event handlers
    search_button.click(
        fn=update_filtered_articles,
        inputs=[search_input, master_articles_state],
        # outputs=[filtered_checkbox, master_articles_state]
        outputs=[filtered_checkbox, filtered_checkbox_state]
    )
    
    filtered_checkbox.select(
        fn=update_filtered_checkbox_articles,
        inputs=[filtered_checkbox, filtered_checkbox_state, master_articles_state],
        outputs=[selected_checkbox, filtered_checkbox_state, master_articles_state]
    )
    
    selected_checkbox.select(
        fn=update_selected_checkbox_articles,
        inputs=[selected_checkbox, filtered_checkbox_state, master_articles_state],
        outputs=[selected_checkbox, filtered_checkbox, master_articles_state, filtered_checkbox_state]
    )
    print("Block")
    

if __name__ == "__main__":
    demo.launch()
```

## `CheckboxGroupMarkdown`

### Initialization

<table>
<thead>
<tr>
<th align="left">name</th>
<th align="left" style="width: 25%;">type</th>
<th align="left">default</th>
<th align="left">description</th>
</tr>
</thead>
<tbody>
<tr>
<td align="left"><code>choices</code></td>
<td align="left" style="width: 25%;">

```python
list[dict] | None
```

</td>
<td align="left"><code>None</code></td>
<td align="left">A list of string or numeric options to select from. An option can also be a tuple of the form (name, value), where name is the displayed name of the checkbox button and value is the value to be passed to the function, or returned by the function.</td>
</tr>

<tr>
<td align="left"><code>value</code></td>
<td align="left" style="width: 25%;">

```python
Sequence[str | float | int]
    | str
    | float
    | int
    | Callable
    | None
```

</td>
<td align="left"><code>None</code></td>
<td align="left">Default selected list of options. If a single choice is selected, it can be passed in as a string or numeric type. If callable, the function will be called whenever the app loads to set the initial value of the component.</td>
</tr>

<tr>
<td align="left"><code>type</code></td>
<td align="left" style="width: 25%;">

```python
ChoiceType
```

</td>
<td align="left"><code>"value"</code></td>
<td align="left">Type of value to be returned by component. "value" returns the list of strings of the choices selected, "index" returns the list of indices of the choices selected.</td>
</tr>

<tr>
<td align="left"><code>buttons</code></td>
<td align="left" style="width: 25%;">

```python
Optional[List[str]]
```

</td>
<td align="left"><code>None</code></td>
<td align="left">None</td>
</tr>

<tr>
<td align="left"><code>label</code></td>
<td align="left" style="width: 25%;">

```python
str | None
```

</td>
<td align="left"><code>None</code></td>
<td align="left">the label for this component, displayed above the component if `show_label` is `True` and is also used as the header if there are a table of examples for this component. If None and used in a `gr.Interface`, the label will be the name of the parameter this component corresponds to.</td>
</tr>

<tr>
<td align="left"><code>info</code></td>
<td align="left" style="width: 25%;">

```python
str | None
```

</td>
<td align="left"><code>None</code></td>
<td align="left">additional component description, appears below the label in smaller font. Supports markdown / HTML syntax.</td>
</tr>

<tr>
<td align="left"><code>every</code></td>
<td align="left" style="width: 25%;">

```python
Timer | float | None
```

</td>
<td align="left"><code>None</code></td>
<td align="left">Continously calls `value` to recalculate it if `value` is a function (has no effect otherwise). Can provide a Timer whose tick resets `value`, or a float that provides the regular interval for the reset Timer.</td>
</tr>

<tr>
<td align="left"><code>inputs</code></td>
<td align="left" style="width: 25%;">

```python
Component | Sequence[Component] | set[Component] | None
```

</td>
<td align="left"><code>None</code></td>
<td align="left">Components that are used as inputs to calculate `value` if `value` is a function (has no effect otherwise). `value` is recalculated any time the inputs change.</td>
</tr>

<tr>
<td align="left"><code>show_label</code></td>
<td align="left" style="width: 25%;">

```python
bool | None
```

</td>
<td align="left"><code>None</code></td>
<td align="left">If True, will display label.</td>
</tr>

<tr>
<td align="left"><code>container</code></td>
<td align="left" style="width: 25%;">

```python
bool
```

</td>
<td align="left"><code>True</code></td>
<td align="left">If True, will place the component in a container - providing some extra padding around the border.</td>
</tr>

<tr>
<td align="left"><code>scale</code></td>
<td align="left" style="width: 25%;">

```python
int | None
```

</td>
<td align="left"><code>None</code></td>
<td align="left">Relative width compared to adjacent Components in a Row. For example, if Component A has scale=2, and Component B has scale=1, A will be twice as wide as B. Should be an integer.</td>
</tr>

<tr>
<td align="left"><code>min_width</code></td>
<td align="left" style="width: 25%;">

```python
int
```

</td>
<td align="left"><code>160</code></td>
<td align="left">Minimum pixel width, will wrap if not sufficient screen space to satisfy this value. If a certain scale value results in this Component being narrower than min_width, the min_width parameter will be respected first.</td>
</tr>

<tr>
<td align="left"><code>interactive</code></td>
<td align="left" style="width: 25%;">

```python
bool | None
```

</td>
<td align="left"><code>None</code></td>
<td align="left">If True, choices in this checkbox group will be checkable; if False, checking will be disabled. If not provided, this is inferred based on whether the component is used as an input or output.</td>
</tr>

<tr>
<td align="left"><code>visible</code></td>
<td align="left" style="width: 25%;">

```python
bool
```

</td>
<td align="left"><code>True</code></td>
<td align="left">If False, component will be hidden.</td>
</tr>

<tr>
<td align="left"><code>elem_id</code></td>
<td align="left" style="width: 25%;">

```python
str | None
```

</td>
<td align="left"><code>None</code></td>
<td align="left">An optional string that is assigned as the id of this component in the HTML DOM. Can be used for targeting CSS styles.</td>
</tr>

<tr>
<td align="left"><code>elem_classes</code></td>
<td align="left" style="width: 25%;">

```python
list[str] | str | None
```

</td>
<td align="left"><code>None</code></td>
<td align="left">An optional list of strings that are assigned as the classes of this component in the HTML DOM. Can be used for targeting CSS styles.</td>
</tr>

<tr>
<td align="left"><code>render</code></td>
<td align="left" style="width: 25%;">

```python
bool
```

</td>
<td align="left"><code>True</code></td>
<td align="left">If False, component will not render be rendered in the Blocks context. Should be used if the intention is to assign event listeners now but render the component later.</td>
</tr>

<tr>
<td align="left"><code>key</code></td>
<td align="left" style="width: 25%;">

```python
int | str | None
```

</td>
<td align="left"><code>None</code></td>
<td align="left">if assigned, will be used to assume identity across a re-render. Components that have the same key across a re-render will have their value preserved.</td>
</tr>
</tbody></table>


### Events

| name | description |
|:-----|:------------|
| `change` | Triggered when the value of the CheckboxGroupMarkdown changes either because of user input (e.g. a user types in a textbox) OR because of a function update (e.g. an image receives a value from the output of an event trigger). See `.input()` for a listener that is only triggered by user input. |
| `input` | This listener is triggered when the user changes the value of the CheckboxGroupMarkdown. |
| `select` | Event listener for when the user selects or deselects the CheckboxGroupMarkdown. Uses event data gradio.SelectData to carry `value` referring to the label of the CheckboxGroupMarkdown, and `selected` to refer to state of the CheckboxGroupMarkdown. See EventData documentation on how to use this event data |



### User function

The impact on the users predict function varies depending on whether the component is used as an input or output for an event (or both).

- When used as an Input, the component only impacts the input signature of the user function.
- When used as an output, the component only impacts the return signature of the user function.

The code snippet below is accurate in cases where the component is used as both an input and an output.

- **As output:** Is passed, passes the list of checked checkboxes as a `list[str | int | float]` or their indices as a `list[int]` into the function, depending on `type`.
- **As input:** Should return, expects a `list[str | int | float]` of values or a single `str | int | float` value, the checkboxes with these values are checked.

 ```python
 def predict(
     value: typing.Union[list[str], list[int], list[dict]][
    list[str], list[int], list[dict]
]
 ) -> list[str | int | float] | str | int | float | None:
     return value
 ```
 
